package polyglot.ext.polyj.ast;

import polyglot.ast.*;
import polyglot.types.*;
import polyglot.visit.*;
import polyglot.util.*;

import polyglot.ext.jl.ast.*;

import polyglot.ext.polyj.visit.*;

public class AmbNameParam_c extends Node_c implements AmbNameParam
{
    /**
      * The qualified name of this parameter.
      * (Why don't we use a String?  Because we need
      * to use Disamb, which requires the data structures
      * that an AmbReceiver handily encapsulates.)
      */
    protected AmbReceiver name;

    public AmbNameParam_c(Position pos, AmbReceiver name) {
        super(pos);
        this.name = name;
    }

    public AmbReceiver name() {
        return name;
    }

    public AmbNameParam name(AmbReceiver name) {
        AmbNameParam_c n = (AmbNameParam_c) copy();
        n.name = name;
        return n;
    }

    public Object parameter() {
        return name;
    }

    public String toString() {
        return name + "{amb}";
    }

    protected AmbNameParam reconstruct(AmbReceiver name) {
        if (this.name != name) {
            AmbNameParam_c n = (AmbNameParam_c) copy();
            n.name = name;
            return n;
        }
        return this;
    }

    public Node visitChildren(NodeVisitor v) {
        AmbReceiver newName = (AmbReceiver) visitChild(name, v);
        return reconstruct(newName);
    }

    public NodeVisitor disambiguateEnter(AmbiguityRemover ar) throws SemanticException {
        return ar.bypass(name);
    }

    public Node disambiguate(AmbiguityRemover ar) throws SemanticException {
        // Must override since name.visit(v) will be unambiguous.
        Context c = ar.context();
        PolyJNodeFactory nf = (PolyJNodeFactory) ar.nodeFactory();

        Prefix disambPrefix = (Prefix) name.visitChild(name.prefix(), ar);

        Node n = nf.disamb().disambiguate(name, ar, position(),
                                          disambPrefix, name.name());

        if (n instanceof TypeNode) {
            return nf.TypeParam(position(), (TypeNode) n);
        } else if (n instanceof Expr) {
            return nf.ExprParam(position(), (Expr) n);
        } else {
            throw new SemanticException("Illegal parameter");
        }
    }

    public void prettyPrint(CodeWriter w, PrettyPrinter tr) {
        print(name, w, tr);
    }

    public void translate(CodeWriter w, Translator tr) {
        throw new InternalCompilerError("Cannot translate " + this);
    }
}
